/* ************************************************************************** */
/*                                                                            */
/*                                                        :::      ::::::::   */
/*   infile.c                                           :+:      :+:    :+:   */
/*                                                    +:+ +:+         +:+     */
/*   By: cchauvet <cchauvet@student.42angoulem      +#+  +:+       +#+        */
/*                                                +#+#+#+#+#+   +#+           */
/*   Created: 2023/02/15 17:52:10 by cchauvet          #+#    #+#             */
/*   Updated: 2023/02/20 13:05:43 by starnakin        ###   ########.fr       */
/*                                                                            */
/* ************************************************************************** */

#include "libftx/libftx.h"
#include "minishell.h"
#include "utils/utils.h"

static int	ft_infile_is_valid(const char *line)
{
	char	**tab;
	size_t	i;

	tab = ft_split_quoted(line, ' ');
	if (tab == NULL)
	{
		ft_eprintf("minishell: malloc failed\n");
		return (0);
	}
	if (tab[0] == NULL)
		return (1);
	i = 0;
	while (tab[i + 1] != NULL)
		i++;
	if (tab[i][0] == '<')
	{
		ft_eprintf("minishell: %s: must be followed by an infile\n", tab[i]);
		ft_freer_tab_ultimate(1, tab);
		return (0);
	}
	ft_freer_tab_ultimate(1, tab);
	return (1);
}

static int	ft_get_infile(const char *line)
{
	size_t	i;
	int		fd;
	char	**tab;

	tab = ft_split_quoted(line, ' ');
	if (tab == NULL)
	{
		ft_eprintf("minishell: malloc failed\n");
		return (-2);
	}
	fd = 0;
	i = 0;
	while (tab[i + 1] != NULL)
	{
		if (tab[i][0] == '<')
			if (fd != 0)
				close(fd);
		if (ft_strcmp("<", tab[i]) == 0)
			fd = ft_file_is_readable(ft_quote_remover(tab[i + 1]));
		else if (ft_strcmp("<<", tab[i]) == 0)
			fd = ft_heredoc(tab[i + 1]);
		i++;
	}
	ft_freer_tab_ultimate(1, tab);
	return (fd);
}

static int	ft_remove_infile(char *line)
{
	size_t	i;
	size_t	y;
	char	**tab;

	tab = ft_split_quoted(line, ' ');
	if (tab == NULL)
	{
		ft_eprintf("minishell: malloc failed\n");
		return (1);
	}
	i = 0;
	y = 0;
	while (tab[i] != NULL)
	{
		if (tab[i][0] == '<')
			ft_strshift(line + y,
			(-1) * (ft_strlen(tab[i]) + ft_strlen(tab[i + 1]) + 1 + (line[ft_strlen(tab[i]) + ft_strlen(tab[i + 1]) + 1] != '\0')));
		else
			y = y + ft_strlen(tab[i]) + (y != 0);
		i++;
	}
	ft_freer_tab_ultimate(1, tab);
	return (0);
}

int	ft_infile(char *line)
{
	int	fd;

	if (ft_infile_is_valid(line) == 0)
		return (-2);
	fd = ft_get_infile(line);
	if (fd == -2)
		return (-2);
	ft_remove_infile(line);
	return (fd);
}
